@@ -25,6 +25,7 @@ CmdExtract::CmdExtract(CommandData *Cmd)
 #ifdef RAR_SMP
   Unp->SetThreads(Cmd->Threads);
 #endif
+  Unp->AllowLargePages(Cmd->UseLargePages);
 }
 
 
@@ -67,7 +68,7 @@ void CmdExtract::DoExtract()
   }
 
   Cmd->ArcNames.Rewind();
-  while (Cmd->GetArcName(ArcName))
+  for (uint ArcCount=0;Cmd->GetArcName(ArcName);ArcCount++)
   {
     if (Cmd->ManualPassword)
       Cmd->Password.Clean(); // Clean user entered password before processing next archive.
@@ -76,6 +77,12 @@ void CmdExtract::DoExtract()
     UseExactVolName=false; // Must be reset here, not in ExtractArchiveInit().
     while (true)
     {
+      // 2025.05.11: Add the empty line between tested archives here instead
+      // of printing two leading "\n" in "\n\nExtracting from", which caused
+      // the extra empty line after the copyright message.
+      if (ArcCount>0)
+        mprintf(L"\n");
+
       EXTRACT_ARC_CODE Code=ExtractArchive();
       if (Code!=EXTRACT_ARC_REPEAT)
         break;
@@ -112,6 +119,15 @@ void CmdExtract::DoExtract()
 
 void CmdExtract::ExtractArchiveInit(Archive &Arc)
 {
+  if (Cmd->Command[0]=='T' || Cmd->Command[0]=='I')
+    Cmd->Test=true;
+
+#ifdef PROPAGATE_MOTW
+  // Invoke here, so it is also supported by unrar.dll.
+  if (!Cmd->Test && Cmd->MotwList.ItemsCount()>0)
+    Arc.Motw.ReadZoneIdStream(Arc.FileName,Cmd->MotwAllFields);
+#endif
+
   DataIO.AdjustTotalArcSize(&Arc);
 
   FileCount=0;
@@ -178,12 +194,18 @@ EXTRACT_ARC_CODE CmdExtract::ExtractArchive()
     }
 #endif
 
-    mprintf(St(MNotRAR),ArcName.c_str());
-
+    bool RarExt=false;
 #ifndef SFX_MODULE
-    if (CmpExt(ArcName,L"rar"))
+    RarExt=CmpExt(ArcName,L"rar");
 #endif
-      ErrHandler.SetErrorCode(RARX_WARNING);
+
+    if (RarExt)
+      uiMsg(UIERROR_BADARCHIVE,ArcName); // Non-archive .rar file.
+    else
+      mprintf(St(MNotRAR),ArcName.c_str()); // Non-archive not .rar file, likely in "rar x *.*".
+
+    if (RarExt)
+      ErrHandler.SetErrorCode(RARX_BADARC);
     return EXTRACT_ARC_NEXT;
   }
 
@@ -254,9 +276,6 @@ EXTRACT_ARC_CODE CmdExtract::ExtractArchive()
 
   ExtractArchiveInit(Arc);
 
-  if (Cmd->Command[0]=='T' || Cmd->Command[0]=='I')
-    Cmd->Test=true;
-
 
   if (Cmd->Command[0]=='I')
   {
@@ -494,7 +513,7 @@ bool CmdExtract::ExtractCurrentFile(Archive &Arc,size_t HeaderSize,bool &Repeat)
           DestFileName=!Cmd->TempPath.empty() ? Cmd->TempPath:Cmd->ExtrPath;
           AddEndSlash(DestFileName);
           DestFileName+=L"__tmp_reference_source_";
-          MkTemp(DestFileName);
+          MkTemp(DestFileName,nullptr);
           MatchedRef.TmpName=DestFileName;
         }
         RefTarget=true; // Need it even for 't' to test the reference source.
@@ -553,6 +572,7 @@ bool CmdExtract::ExtractCurrentFile(Archive &Arc,size_t HeaderSize,bool &Repeat)
       return !Arc.Solid; // Can try extracting next file only in non-solid archive.
     }
 
+#ifndef RAR_NOCRYPT  // For rarext.dll, Setup.SFX and unrar_nocrypt.dll.
     if (Arc.FileHead.Encrypted)
     {
       RarCheckPassword CheckPwd;
@@ -579,9 +599,9 @@ bool CmdExtract::ExtractCurrentFile(Archive &Arc,size_t HeaderSize,bool &Repeat)
         // Set a password before creating the file, so we can skip creating
         // in case of wrong password.
         SecPassword FilePassword=Cmd->Password;
-  #if defined(_WIN_ALL) && !defined(SFX_MODULE)
+#if defined(_WIN_ALL) && !defined(SFX_MODULE)
         ConvertDosPassword(Arc,FilePassword);
-  #endif
+#endif
 
         byte PswCheck[SIZE_PSWCHECK];
         bool EncSet=DataIO.SetEncryption(false,Arc.FileHead.CryptMethod,
@@ -609,16 +629,16 @@ bool CmdExtract::ExtractCurrentFile(Archive &Arc,size_t HeaderSize,bool &Repeat)
 
             // Avoid new requests for unrar.dll to prevent the infinite loop
             // if app always returns the same password.
-  #ifndef RARDLL
+#ifndef RARDLL
             continue; // Request a password again.
-  #endif
+#endif
           }
-  #ifdef RARDLL
+#ifdef RARDLL
           // If we already have ERAR_EOPEN as result of missing volume,
           // we should not replace it with less precise ERAR_BAD_PASSWORD.
           if (Cmd->DllError!=ERAR_EOPEN)
             Cmd->DllError=ERAR_BAD_PASSWORD;
-  #endif
+#endif
           ErrHandler.SetErrorCode(RARX_BADPWD);
           ExtrFile=false;
         }
@@ -627,6 +647,7 @@ bool CmdExtract::ExtractCurrentFile(Archive &Arc,size_t HeaderSize,bool &Repeat)
     }
     else
       DataIO.SetEncryption(false,CRYPT_NONE,NULL,NULL,NULL,0,NULL,NULL);
+#endif // RAR_NOCRYPT
 
     // Per file symlink conversion flag. Can be turned off in unrar.dll.
     bool CurConvertSymlinkPaths=ConvertSymlinkPaths;
@@ -688,7 +709,29 @@ bool CmdExtract::ExtractCurrentFile(Archive &Arc,size_t HeaderSize,bool &Repeat)
           // any overwrite prompts.
           if (!CheckWinLimit(Arc,ArcFileName))
             return false;
-          ExtrFile=ExtrCreateFile(Arc,CurFile);
+
+          // Read+write mode is required to set "Compressed" attribute.
+          // Other than that prefer the write only mode to avoid
+          // OpenIndiana NAS problem with SetFileTime and read+write files.
+#if defined(_WIN_ALL) && !defined(SFX_MODULE)
+          bool Compressed=Cmd->SetCompressedAttr &&
+               (Arc.FileHead.FileAttr & FILE_ATTRIBUTE_COMPRESSED)!=0;
+          bool WriteOnly=!Compressed;
+#else
+          bool WriteOnly=true;
+#endif
+
+          ExtrFile=ExtrCreateFile(Arc,CurFile,WriteOnly);
+
+#if defined(_WIN_ALL) && !defined(SFX_MODULE)
+          // 2024.03.12: Set early to compress written data immediately.
+          // For 10 GB text file it was ~1.5x faster than when set after close.
+
+          if (ExtrFile && Compressed)
+            SetFileCompression(CurFile.GetHandle(),true);
+
+#endif
+
         }
 
     if (!ExtrFile && Arc.Solid)
@@ -968,6 +1011,9 @@ bool CmdExtract::ExtractCurrentFile(Archive &Arc,size_t HeaderSize,bool &Repeat)
           if (Preallocated>0 && (BrokenFile || DataIO.CurUnpWrite!=Preallocated))
             CurFile.Truncate();
 
+#ifdef PROPAGATE_MOTW
+          Arc.Motw.CreateZoneIdStream(DestFileName,Cmd->MotwList);
+#endif
 
           CurFile.SetOpenFileTime(
             Cmd->xmtime==EXTTIME_NONE ? NULL:&Arc.FileHead.mtime,
@@ -987,9 +1033,6 @@ bool CmdExtract::ExtractCurrentFile(Archive &Arc,size_t HeaderSize,bool &Repeat)
         if (SetAttr)
         {
 #if defined(_WIN_ALL) && !defined(SFX_MODULE)
-          if (Cmd->SetCompressedAttr &&
-              (Arc.FileHead.FileAttr & FILE_ATTRIBUTE_COMPRESSED)!=0)
-            SetFileCompression(DestFileName,true);
           if (Cmd->ClearArc)
             Arc.FileHead.FileAttr&=~FILE_ATTRIBUTE_ARCHIVE;
 #endif
@@ -1406,7 +1449,7 @@ void CmdExtract::ExtrCreateDir(Archive &Arc,const std::wstring &ArcFileName)
 }
 
 
-bool CmdExtract::ExtrCreateFile(Archive &Arc,File &CurFile)
+bool CmdExtract::ExtrCreateFile(Archive &Arc,File &CurFile,bool WriteOnly)
 {
   bool Success=true;
   wchar Command=Cmd->Command[0];
@@ -1417,9 +1460,7 @@ bool CmdExtract::ExtrCreateFile(Archive &Arc,File &CurFile)
   if ((Command=='E' || Command=='X') && !Cmd->Test)
   {
     bool UserReject;
-    // Specify "write only" mode to avoid OpenIndiana NAS problems
-    // with SetFileTime and read+write files.
-    if (!FileCreate(Cmd,&CurFile,DestFileName,&UserReject,Arc.FileHead.UnpSize,&Arc.FileHead.mtime,true))
+    if (!FileCreate(Cmd,&CurFile,DestFileName,&UserReject,Arc.FileHead.UnpSize,&Arc.FileHead.mtime,WriteOnly))
     {
       Success=false;
       if (!UserReject)
@@ -1479,10 +1520,16 @@ bool CmdExtract::CheckUnpVer(Archive &Arc,const std::wstring &ArcFileName)
   if (Arc.FileHead.Method==0)
     WrongVer=false;
 
+  // Can't unpack the unknown encryption even for stored files.
+  if (Arc.FileHead.CryptMethod==CRYPT_UNKNOWN)
+    WrongVer=true;
+
   if (WrongVer)
   {
     ErrHandler.UnknownMethodMsg(Arc.FileName,ArcFileName);
-    uiMsg(UIERROR_NEWERRAR,Arc.FileName);
+    // No need to suggest a new version if it is just a broken archive.
+    if (!Arc.BrokenHeader)
+      uiMsg(UIERROR_NEWERRAR,Arc.FileName);
   }
   return !WrongVer;
 }
@@ -1565,8 +1612,10 @@ void CmdExtract::AnalyzeArchive(const std::wstring &ArcName,bool Volume,bool New
 
         if (!Arc.FileHead.SplitBefore)
         {
-          if (!MatchFound && !Arc.FileHead.Solid) // Can start extraction from here.
+          if (!MatchFound && !Arc.FileHead.Solid && !Arc.FileHead.Dir &&
+              Arc.FileHead.RedirType==FSREDIR_NONE && Arc.FileHead.Method!=0)
           {
+            // Can start extraction from here.
             // We would gain nothing and unnecessarily complicate extraction
             // if we set StartName for first volume or StartPos for first
             // archived file.
